#ifndef IMAGE_H
#define IMAGE_H
// image.h : Defines the entry point for the console application.
// bmp save routines.  image merge routines. jpg save routines
// uses jpeg-6b driver.
#ifdef SENSORAY_OS_LINUX
#include "types2226.h"
#else 
#include <ctype.h>
#endif


#define MAX_IMAGE_HEIGHT (540 *2)   // HD
#define MAX_IMAGE_WIDTH  1920       // HD

#ifdef __cplusplus
extern "C" {
#endif
/** image information
 *  @param width width of image (bytes)
 *  @param height height of image (bytes)
 */
typedef struct 
{
    int stride;
    int width;
    int height;
} image_info_t;

#if 0
// future use
	/** converts image buffer to JPG file specified
* @param image_buffer is a pointer to the image
* @param outfilename is the full path name of the resulting file
* @param info is the image info structure
* @param qual quality of jpeg.  25-100 where 100 is highest quality(largest file size)
* @return 0 on success, -ve on fail
*/
//int image_to_jpg( const unsigned char *imagebuffer, const char *outfilename, image_info_t *info, int qual );
#endif

/** saves an image to bmp file
 *  @param filename is the full file path to save( minus the extension)
 *  @param *image, pointer to image data in Y Cr Cb format
 *  @param type is the type( 0-- BMP,  1--ppm)
 *  @return 0 on success, -1 if file invalid
*/
int save_image_uncompressed(const unsigned char *image, const char *szFilename, image_info_t *info, int type);


/** Converts 8-bit YCrCb image to 8-bit RGB image in memory using BT.601 color space
 *  @param in pointer to input image
 *  @param out pointer to output image
 *  @param height height of image
 *  @param width width of image
 *  @return 0 on success
 */
int YCrCb2RGB_BT601( const unsigned char *in, unsigned char *out, int height, int width);

/** Converts 8-bit YCrCb image to 8-bit RGB image in memory using BT.709 color space
 *  @param in pointer to input image
 *  @param out pointer to output image
 *  @param height height of image
 *  @param width width of image
 *  @return 0 on success
 */
int YCrCb2RGB_BT709( const unsigned char *in, unsigned char *out, int height, int width);

// all functions below must have the same calling protocol (mergefunc_t in mid2246func.h)

/** Invert field zero
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1
 *  @param h1 height of field 1
 *  @param w  width of fields( full width in bytes)
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int inv_fieldzero( unsigned char *newf, int size,  unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH);

/** duplicate fields
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1
 *  @param h1 height of field 1
 *  @param w  width of fields( full width in bytes)
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int dup_fields( unsigned char *newf, int size, unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH);
/** merge fields
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1 (unused)
 *  @param h1 height of field 1  (unused)
 *  @param w  width of fields in bytes
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int merge_fields( unsigned char *newf, int size, unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH);


/** interpolate fields
 *  @param newf pointer to allocated space for new field( must be large enough
 *  @param size size of newf
 *  @param f0 pointer to field 0
 *  @param h0 height of field 0
 *  @param f1 pointer to field 1 
 *  @param h1 height of field 1  
 *  @param w  width of fields in bytes
 *  @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
 *  @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
 *  @return 0 on success, -1 otherwise
 */
int inter_fields( unsigned char *newf, int size, unsigned char *f0, int h0, unsigned char *f1, int h1, int w, int *pRetW, int *pRetH);

/** Converts RGB to YCrCb using 8-bit YCrCb to 8-bit RGB 
 * and the BT601 color space definition.
 * Y  =  0.257*R + 0.504*G + 0.098*B + 16
 * Cr =  0.439*R - 0.368*G - 0.071*B + 128
 * Cb = -0.148*R - 0.291*G + 0.439*B + 128
 *
 * or using integer math.
 *
 * Y  = ( 16842R + 33030G +  6422B + 1048576 ) / 65536
 * Cr = ( 28770R - 24117G -  4653B + 8388608 ) / 65536
 * Cb = ( -9699R - 19070G + 28770B + 8388608 ) / 65536
 *
 *  @param R red value
 *  @param G green value
 *  @param B blue value
 *  @return YCrCb triplet short
 */
unsigned short RGB2YCrCb16_BT601( int R, int G, int B );

/** Converts RGB to YCrCb using 8-bit YCrCb to 8-bit RGB 
 * and the BT709 color space definition.
 * Y  =  0.257*R + 0.504*G + 0.098*B + 16
 * Cr =  0.439*R - 0.368*G - 0.071*B + 128
 * Cb = -0.148*R - 0.291*G + 0.439*B + 128
 *
 * or using integer math.
 *
 * Y  = ( 16842R + 33030G +  6422B + 1048576 ) / 65536
 * Cr = ( 28770R - 24117G -  4653B + 8388608 ) / 65536
 * Cb = ( -9699R - 19070G + 28770B + 8388608 ) / 65536
 *
 *  @param R red value
 *  @param G green value
 *  @param B blue value
 *  @return YCrCb triplet short
 */

int yuy2_to_rgb_mem_bt601( const unsigned char *pin, unsigned char *pout, const unsigned char *pend, int stride);


unsigned short RGB2YCrCb16_BT709( int R, int G, int B );
#ifdef __cplusplus
}
#endif

#endif
