#include <sys/ioctl.h>
#include <sys/time.h>
#ifdef SENSORAY_OS_LINUX
#include <endian.h>
#endif
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "ovlgen.h"
#include "ovlmid.h"
#define BASE_VIDIOC_PRIVATE 192
#include "s2226ioctl.h"
//#include "fpga2226reg.h"
#include "fpga2226.h"
#include "dh2226.h"
#include "image.h"


#define VERSION_ARM_ADDRDATA 0x58
// Macros to update overlay.  Uses either many driver calls, or write to m_overlayMem first
// and then does a burst transfer of type addr/data.

// WriteIndexByte and some other routines must be protected 
// by a critical section(any) to make it atomic.
// Currently, these calls are protected by m_csOvl in mid2226.cpp

#define OVLMEM m_ovl[idx].m_overlayMem
#define OVLPOS m_ovl[idx].m_overlayPos
#define USE_ADDRDATA (m_ovl[idx].m_arm_fw >= VERSION_ARM_ADDRDATA)
#define WRITE_INDEXED_BYTE( bOvl, reg1, val1)	\
 {if (bOvl) \
    WriteIndexedByteMem(OVLMEM, &OVLPOS, reg1,val1); \
 else \
    WriteIndexedByte(handle, reg1,val1);\
 }

#define WRITE_INDEXED_BYTE_DLL(bOvl, reg1, val1)	\
{if (bOvl) \
    WriteIndexedByteMem(OVLMEM, &OVLPOS, reg1,val1);\
else \
     WriteIndexedByte(handle, reg1,val1);\
}     

#define SET_OVERLAY_WINDOW_POSITION(bOvl, idx1, reg1, top1,bot1,left1,right1)\
{if (bOvl) \
    SetOverlayWindowPositionMem(OVLMEM, &OVLPOS, idx1, reg1,top1,bot1,left1,right1);\
else \
    SetOverlayWindowPosition(handle,idx1, reg1, top1,bot1,left1,right1);\
}

#define SET_OVERLAY_WINDOW_POSITION_DLL(bOvl, idx1, reg1, top1,bot1,left1,right1)\
{if (bOvl) \
    SetOverlayWindowPositionMem(OVLMEM, &OVLPOS, idx1, reg1,top1,bot1,left1,right1);\
else \
    SetOverlayWindowPosition(handle, idx1, reg1, top1,bot1,left1,right1);\
}
 
#define SET_BLACK_WINDOW_COLOR(bOvl,col1)\
{if (bOvl)\
    SetBlackWindowColorMem(OVLMEM, &OVLPOS, col1);\
else \
    SetBlackWindowColor(handle, col1);\
}

#define SET_BLACK_WINDOW_POSITION(bOvl, idx1, reg1, top1,bot1,left1,right1)\
{if (bOvl) \
    SetBlackWindowPositionMem(OVLMEM, &OVLPOS, idx1, reg1,top1,bot1,left1,right1);\
else \
    SetBlackWindowPosition(handle, idx1, reg1,top1,bot1,left1,right1);\
}


#define MAX_BOARDS 16 // increase if using more than 16 2226's 


struct s2226_ovl {
    OVLHAN    m_hOvl;                // handle to overlay
    int       m_bOvlChanged;         // boolean if user changes bitmap size, position, font or text.
    overlay_t m_overlay[OVLWINDOWS]; // overlay array
    backwnd_t m_backgnd[OVLWINDOWS]; // background structures
// current field dimensions
    int  m_imageHeight;
    int  m_imageWidth;
    int  m_field0Height;
    int  m_field1Height;
    // updates overlay window positions using FIFO write
    // versus Control channel(multiple single VX_ commands)
    // for faster update.  
    char m_overlayMem[2048];
    int  m_overlayPos;

    WORD  *m_bitmap; // Pointer to Overlay bitmap
    int m_input; // see dh2226.h
    int m_mergeType;
    int m_arm_fw;
};


//static int m_handlelist[MAX_BOARDS];
static struct s2226_ovl m_ovl[MAX_BOARDS] = 
  { {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
    {.m_arm_fw = 0},
  };



static int SetOverlayBackground(int WinIndex, RECT *pos, int color, int regionmask, int group, int idx);

static int get_idx_from_handle(int handle)
{
    return 0;
}


static int s2226_get_arm_ver(int handle)
{
  int ver;
  int rc;
  rc = ioctl(handle, S2226_VIDIOC_ARM_VER, &ver);
  if (rc)
      ver = 0;
  return ver;
}

static int s2226_get_input(int handle, int *input)
{
  int rc;
  rc = ioctl(handle, S2226_VIDIOC_GET_INPUT, input);
  return rc;
}

static int s2226_lock_overlay(int handle)
{
    int rc;
    rc = ioctl(handle, S2226_VIDIOC_LOCK_OVERLAY, 0);
    return rc;
}

static int s2226_unlock_overlay(int handle)
{
    int rc;
    rc = ioctl(handle, S2226_VIDIOC_UNLOCK_OVERLAY, 0);
    return rc;
}


static int InitOverlay(int idx)
{
    int WinIndex;
    char fntFile[MAX_PATH];
    RECT zerorect = {0,0,0,0};
    m_ovl[idx].m_hOvl = CreateOverlay( 1);
    if (m_ovl[idx].m_hOvl == NULL) {
      return -1;
    }

#if defined (SENSORAY_OS_LINUX)
    strcpy(fntFile, "arial.ttf");
#else
#error "windows"
    GetWindowsDirectory( winDir, MAX_PATH);
    int tempsz = _sntprintf(temp, MAX_PATH, _T("%s\\fonts\\arial.ttf"), winDir);
    // Convert an to ASCII string = UTF-8 (Multi-byte format)
	WideCharToMultiByte(CP_UTF8,0,temp,-1,fntFile,MAX_PATH,NULL,NULL);
#endif
    
    for (WinIndex=0; WinIndex < OVLWINDOWS; WinIndex++)
    {
        SetOverlayTextColor( m_ovl[idx].m_hOvl, WinIndex, RGB(255,255,255), RGB(0,0,0) );
        SetOverlayFont( m_ovl[idx].m_hOvl, WinIndex, fntFile, 30, 0 );
        m_ovl[idx].m_overlay[WinIndex].regionmask = -1;
        m_ovl[idx].m_overlay[WinIndex].group = -1;
        m_ovl[idx].m_overlay[WinIndex].pos = zerorect;
        SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx); // No background for images
        //m_ovl[idx].m_backgnd[WinIndex].group = -1; // Done via SetOverlayBackground()
    }
    m_ovl[idx].m_bOvlChanged = TRUE;
    return 0;
} // InitOverlay()



/** Reloads and resets all overlay parameters
 *  For Text
 *    - SetOverlayFont()  --> moved to SetOverlayTextExt
 *    - SetOverlayTextColor()  --> moved to SetOverlayTextExt
 *    - SetOverlayText()  --> moved to SetOverlayTextExt
 *    - Sets position right and bottom according to rasterized (SetOverlayText) image
 *    - Tracks groups, and set background widths of all members of each group to the max members width.
 * For Images
 *    - SetOverlayPosition()
 *    - SetOverlayImage() --> 
 *    - Deletes background
 */
static int RegenerateOverlay(int idx)
{
    int WinIndex;
    int lasterr = 0;
    RECT rect;
    RECT zerorect = {0,0,0,0};
    int err;
    int curgroup = -1;
    int backcolor;
    int region;
    //int lasttop;
    //#define LASTBOT_INIT 99
    //    int lastbot = LASTBOT_INIT;
    //TODO lastbot is not initializied
    m_ovl[idx].m_bOvlChanged = TRUE;

    // Background windows are generated here
    for ( WinIndex=0; WinIndex < OVLWINDOWS; WinIndex++ ) {
        if (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) {
            if ( m_ovl[idx].m_overlay[WinIndex].type == OVERLAY_TYPE_TEXT) // Text
            {
                int i;
                curgroup  = m_ovl[idx].m_overlay[WinIndex].group;
                region    = m_ovl[idx].m_overlay[WinIndex].regionmask;
                backcolor = m_ovl[idx].m_overlay[WinIndex].ovltext.backcolor;
                
                // find other windows in the same group and adjust width if we are smaller
                for (i=0; i < OVLWINDOWS; i++) {
                    if (m_ovl[idx].m_overlay[i].group == curgroup)
                        if (m_ovl[idx].m_overlay[WinIndex].pos.right < m_ovl[idx].m_overlay[i].pos.right)
                            m_ovl[idx].m_overlay[WinIndex].pos.right = m_ovl[idx].m_overlay[i].pos.right;
                }
                rect = m_ovl[idx].m_overlay[WinIndex].pos;
                
                err = SetOverlayPosition( m_ovl[idx].m_hOvl, WinIndex, rect.left, rect.top);
                if( err !=0) 
                    lasterr = err;
            
                if( backcolor!=0) {
                    SetOverlayBackground(WinIndex, &rect, backcolor, region, curgroup, idx);
                } else {
                    SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx);
                }
            }
            else // Bitmap
            {
                SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx); // No background for images
                err = SetOverlayPosition( m_ovl[idx].m_hOvl, WinIndex, m_ovl[idx].m_overlay[WinIndex].pos.left, m_ovl[idx].m_overlay[WinIndex].pos.top);
                if( err !=0) 
                    lasterr = err;
                
                //PE: do not need to reload the overlay images every time this function is called 
                // err = SetOverlayImage( m_ovl[idx].m_hOvl, WinIndex, m_ovl[idx].m_overlay[WinIndex].path);
                // if( err !=0) 
                    // lasterr = err;
            }
        } // valid sub-window
        else {
            // Invalid window
            SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx);
        }
    } // for WinIndex=0; WinIndex < OVLWINDOWS
    return lasterr;
} // RegenerateOverlay


/** Sets the field dimensions based on input and video system
 *  @param input enumerated 2226 input
 *  @param vidsys enumerated 2226 video system
 *  @return none
 */
static int SetFieldDims(int idx)
{
    int input = m_ovl[idx].m_input;
  switch (input) {
  case INPUT_SDI_1080I_50:
  case INPUT_SDI_1080I_60:
  case INPUT_SDI_1080I_5994:
  case INPUT_SDI_1080I_60_CB:
  case INPUT_SDI_1080I_50_CB:
  case INPUT_H51_HD_1080I_60:
  case INPUT_H51_HD_1080I_50:
  case INPUT_H51_HD_1080I_5994:
    m_ovl[idx].m_imageHeight  = 540*2;
    m_ovl[idx].m_imageWidth   = 1920;
    m_ovl[idx].m_field0Height = 540;
    m_ovl[idx].m_field1Height = 540;
    break;
  case INPUT_SDI_720P_5994:
  case INPUT_SDI_720P_60:
  case INPUT_SDI_720P_60_CB:
  case INPUT_H51_HD_720P_60:
  case INPUT_H51_HD_720P_5994:
  case INPUT_SDI_720P_50:
  case INPUT_SDI_720P_50_CB:
  case INPUT_H51_HD_720P_50:
    m_ovl[idx].m_imageHeight  = 720;
    m_ovl[idx].m_imageWidth   = 1280;
    m_ovl[idx].m_field0Height = 720;
    m_ovl[idx].m_field1Height = 0;
    break;
  case INPUT_COMP0_480I:
  case INPUT_SVIDEO0_480I:
  case INPUT_SDI_480I:
  case INPUT_SDI_480I_CB:
  case INPUT_COMP1_480I:
  case INPUT_SVIDEO1_480I:
  case INPUT_H51_SD_480I:
    m_ovl[idx].m_imageHeight  = 243+244;
    m_ovl[idx].m_imageWidth   = 720;
    m_ovl[idx].m_field0Height = 244;
    m_ovl[idx].m_field1Height = 243;
    break;
  case INPUT_COMP0_576I:
  case INPUT_SVIDEO0_576I:
  case INPUT_SDI_576I:
  case INPUT_SDI_576I_CB:
  case INPUT_COMP1_576I:
  case INPUT_SVIDEO1_576I:
  case INPUT_H51_SD_576I:
    m_ovl[idx].m_imageHeight  = 288*2;
    m_ovl[idx].m_imageWidth   = 720;
    m_ovl[idx].m_field0Height = 288;
    m_ovl[idx].m_field1Height = 288;
    break;
  default:
      return -1;
      //printf("undefined input %d\n", input);

  }
  return 0;
}


/**
 *  Sets the overlay text
 *
 *  Use if "\n" in string.  
 *  @param xpos  Left position of text
 *  @param ypos  Top position of text
 *  @param pOvl  Text structure containing text and test attributes. 
 *               Embedded \n delimits lines. Each \n will increase the sub-windows used.
 *  @param regionmask  Three LSB bits define which video streams the overlay will be
 *               applied to.
 *  @return -1 on err, -2 on not enough regions, otherwise returns next region available
 */ 

static int SetOverlayTextExt( int xpos, int ypos, overlay_text_t *pOvl, int regionmask, int startindex, int idx)
{
    int  WinIndex;
    int  numlines=1;
    int  line;
    char *pText;
    char *pLines[OVLWINDOWS];
    char tmpbuffer[256];
    //int  maxwidth = 0;
    int  firstfree,next = 0;
    static int groupcount=0;
    int  err, lasterr = 0;
    int board = 0;

    if (*pOvl->text == '\0') {
        return 0; // No text. Don't add it
    }

    // parse the text into lines
    strncpy( tmpbuffer, pOvl->text, 256);

    pText = tmpbuffer;
    pLines[board] = tmpbuffer;
    while( *pText != '\0') {
        if(*pText == '\n') {
            *pText = '\0';
            pLines[numlines++] = ++pText;
            if(numlines >= OVLWINDOWS)
                break;
            if( *pText == '\0') {
                break;
            }
            if( *pText == '\n') {
                continue; // Don't inc pText
            }
        }
        pText++;
    }

    err = -1; // only returns this if no free space at all.
    next      = OVLWINDOWS;
    firstfree = OVLWINDOWS;
    for ( WinIndex=startindex; (WinIndex < OVLWINDOWS) && next == OVLWINDOWS ; WinIndex++) {
        if (m_ovl[idx].m_overlay[WinIndex].regionmask < 0) {
            firstfree = WinIndex;
            err = -2; // in case we don't find an endpoint
            // Groups must be contiguous
            for ( next=firstfree; (next < OVLWINDOWS) ; next++) {
                if (m_ovl[idx].m_overlay[next].regionmask < 0) {
                    if ((next-firstfree)+1 >= numlines) {
                        break; // next=non-zero will drop us out of outer loop
                    }
                    // else continue searching for end point
                } else { // reserved sub-window
                    next = OVLWINDOWS;
                    break; // Contunue searching with new firstfree
                }
            }
        }
    }

    if (firstfree >= OVLWINDOWS)
        return(err);

    line = 0;
    for ( WinIndex=firstfree; WinIndex < firstfree+numlines ; WinIndex++) {
        if (m_ovl[idx].m_overlay[WinIndex].regionmask < 0) { // Find free position (Should never fail)
            m_ovl[idx].m_overlay[WinIndex].ovltext = *pOvl;
            strcpy( m_ovl[idx].m_overlay[WinIndex].ovltext.text, pLines[line]);
            m_ovl[idx].m_overlay[WinIndex].group = groupcount; // Group always unique
            m_ovl[idx].m_overlay[WinIndex].pos.left = xpos;
            m_ovl[idx].m_overlay[WinIndex].pos.top = ypos;
            m_ovl[idx].m_overlay[WinIndex].type = OVERLAY_TYPE_TEXT;
            m_ovl[idx].m_overlay[WinIndex].regionmask = regionmask;
            
            err = SetOverlayFont( m_ovl[idx].m_hOvl, WinIndex, m_ovl[idx].m_overlay[WinIndex].ovltext.fontfile, m_ovl[idx].m_overlay[WinIndex].ovltext.fontsize, 0);
            if( err !=0) 
                lasterr = err;
            err = SetOverlayTextColor( m_ovl[idx].m_hOvl, WinIndex, m_ovl[idx].m_overlay[WinIndex].ovltext.txtcolor, 0);
            if( err !=0) 
                lasterr = err;
            err = SetOverlayText( m_ovl[idx].m_hOvl, WinIndex, m_ovl[idx].m_overlay[WinIndex].ovltext.text);
            if( err !=0) 
                lasterr = err;
            m_ovl[idx].m_overlay[WinIndex].pos.right = xpos + GetOverlayWidth( m_ovl[idx].m_hOvl, WinIndex);
            ypos += GetOverlayHeight( m_ovl[idx].m_hOvl, WinIndex);
            m_ovl[idx].m_overlay[WinIndex].pos.bottom = ypos;
            
            line++;
        }
    }
    groupcount++; // Increment for next text. Group always unique
    if (groupcount == -1) {
        groupcount = 0;
    }

    //m_ovl[idx].m_bOvlChanged = TRUE;
    return lasterr;
}


/** Regenerates the download image from the previously set up overlay windows
  */
int S2226_OverlayRegenerate(int handle)
{
    int idx = get_idx_from_handle(handle);
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        if (InitOverlay(idx) == -1)
            return -1;
    }
    RegenerateOverlay(idx);
    s2226_unlock_overlay(handle);
    return 0;
} // S2226_OverlayRegenerate()

/** Refresh or update the overlay in hardware
  * returns 0 on success.
  */
static int UpdateOverlay(int handle)
{
    int bytes;
    unsigned int WinIndex;
    int Interlaced;
    int idx = get_idx_from_handle(handle);
    int input = m_ovl[idx].m_input;
    
    if (input == -1) {
        return -1;
    }
    if (m_ovl[idx].m_arm_fw == 0) {
        m_ovl[idx].m_arm_fw = s2226_get_arm_ver(handle);
    }

    //int bBulk = 1;// bulk operation( not functional yet)
    if (m_ovl[idx].m_hOvl == NULL) {
      if (InitOverlay(idx) == -1) {
            return -2;
      }
    }

    if (m_ovl[idx].m_bOvlChanged)  {
        //if ((input == INPUT_SDI_720P_50      ) ||
        //    (input == INPUT_SDI_720P_5994    ) ||
        //    (input == INPUT_SDI_720P_60      ) ||
        //    (input == INPUT_H51_HD_720P_50   ) ||
        //    (input == INPUT_H51_HD_720P_5994 ) ||
        //    (input == INPUT_H51_HD_720P_60   ) ||
        //    (input == INPUT_SDI_720P_50_CB   ) ||
        //    (input == INPUT_SDI_720P_5994_CB ) ||
        //    (input == INPUT_SDI_720P_60_CB   )  ) {
        //    Interlaced = 0;
        //}
	switch (input) {
	case INPUT_SDI_720P_5994:
	case INPUT_SDI_720P_60:
	case INPUT_SDI_720P_60_CB:
	case INPUT_H51_HD_720P_60:
	case INPUT_SDI_720P_50:
	case INPUT_SDI_720P_50_CB:	  
	case INPUT_H51_HD_720P_5994:
	case INPUT_H51_HD_720P_50:	  
    case INPUT_SDI_1080P_30_CB:
    case INPUT_SDI_1080P_2997_CB:
    case INPUT_SDI_1080P_24_CB:
    case INPUT_SDI_1080P_2398_CB:
    case INPUT_SDI_720P_24_CB:
    case INPUT_SDI_720P_2398_CB:
    case INPUT_SDI_1080P_30:
    case INPUT_SDI_1080P_2997:
    case INPUT_SDI_1080P_24:
    case INPUT_SDI_1080P_2398:
    case INPUT_SDI_720P_24:
    case INPUT_SDI_720P_2398:
    case INPUT_H51_HD_720P_24:
    case INPUT_H51_HD_720P_2398:
    case INPUT_H51_HD_1080P_24:
    case INPUT_H51_HD_1080P_2398:
	  Interlaced = 0;
	  break;
	default:
	  Interlaced = 1;
	}
        // Make sure width/height set
	if (SetFieldDims(idx) != 0) {

	  return -3;
	}
        // update the bitmap
        m_ovl[idx].m_bitmap = (WORD *) GetOverlayBitmap( m_ovl[idx].m_hOvl, &bytes, m_ovl[idx].m_imageWidth, m_ovl[idx].m_imageHeight, Interlaced);

        // calculate the rectangles
        for ( WinIndex=0; WinIndex < OVLWINDOWS ; WinIndex++) {
            if (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) {
                m_ovl[idx].m_overlay[WinIndex].pos.bottom = m_ovl[idx].m_overlay[WinIndex].pos.top  + GetOverlayHeight( m_ovl[idx].m_hOvl, WinIndex );
                m_ovl[idx].m_overlay[WinIndex].pos.right  = m_ovl[idx].m_overlay[WinIndex].pos.left + GetOverlayWidth ( m_ovl[idx].m_hOvl, WinIndex );
            }
        }
    }

    //if(ValidDrvHandle(handle)) {
    //	    int handle = m_pDev->m_iInstance;
        // Use 2226 Hardware Overlay
        // Set up overlay regions
        s2226_write_data(handle, S2226_REG_OVAUPD, 0x00); // Clr the UpdateOverlay bit
        if(m_ovl[idx].m_bitmap != 0) {
            SetOverlayMem(handle, m_ovl[idx].m_bitmap, &bytes, 1, m_ovl[idx].m_arm_fw);
        }
        // else clear/set regions and backgrounds

        
        if (USE_ADDRDATA) {
            m_ovl[idx].m_overlayPos = 0;
        }
        // Set sub-windows to correct regions now that all values are correct
        for (WinIndex = 0; WinIndex<OVLWINDOWS ; WinIndex++) {
            if (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0 
            && m_ovl[idx].m_overlay[WinIndex].pos.top < m_ovl[idx].m_imageHeight
            && m_ovl[idx].m_overlay[WinIndex].pos.left < m_ovl[idx].m_imageWidth
            && m_ovl[idx].m_overlay[WinIndex].pos.bottom > 0
            && m_ovl[idx].m_overlay[WinIndex].pos.right > 0) {
                SET_OVERLAY_WINDOW_POSITION(USE_ADDRDATA, WinIndex, m_ovl[idx].m_overlay[WinIndex].regionmask, 
                    MAX(0, m_ovl[idx].m_overlay[WinIndex].pos.top),  MIN(m_ovl[idx].m_imageHeight, m_ovl[idx].m_overlay[WinIndex].pos.bottom), 
                    MAX(0, m_ovl[idx].m_overlay[WinIndex].pos.left), MIN(m_ovl[idx].m_imageWidth, m_ovl[idx].m_overlay[WinIndex].pos.right));
            } else {
                SET_OVERLAY_WINDOW_POSITION(USE_ADDRDATA, WinIndex, 0, 0, 0, 0, 0); // Diasble
            }
        }

        for (WinIndex=0 ; WinIndex < OVLWINDOWS ; WinIndex++) {
            if (m_ovl[idx].m_backgnd[WinIndex].group >= 0 // Valid
            && m_ovl[idx].m_overlay[WinIndex].pos.top < m_ovl[idx].m_imageHeight
            && m_ovl[idx].m_overlay[WinIndex].pos.left < m_ovl[idx].m_imageWidth
            && m_ovl[idx].m_overlay[WinIndex].pos.bottom > 0
            && m_ovl[idx].m_overlay[WinIndex].pos.right > 0) { 
                SET_BLACK_WINDOW_POSITION(USE_ADDRDATA,WinIndex, m_ovl[idx].m_backgnd[WinIndex].regionmask, 
                    MAX(0, m_ovl[idx].m_backgnd[WinIndex].pos.top),  MIN(m_ovl[idx].m_imageHeight, m_ovl[idx].m_backgnd[WinIndex].pos.bottom), 
                    MAX(0, m_ovl[idx].m_backgnd[WinIndex].pos.left), MIN(m_ovl[idx].m_imageWidth, m_ovl[idx].m_backgnd[WinIndex].pos.right+3));
                // Bug: Last background sets color
                //SET_BLACK_WINDOW_COLOR(m_ovl[idx].m_overlay[WinIndex].ovltext.backcolor);
            } else {
                SET_BLACK_WINDOW_POSITION(USE_ADDRDATA, WinIndex, 0, 0, 0, 0, 0); // Diasble
            }
        }

        if (USE_ADDRDATA) {
            int i = 0;
            int burst;
            // Burst out the new sub-window position's
            while (OVLPOS > 0) {
                burst = OVLPOS > 498 ? 498 : OVLPOS;
                s2226_write_addr_data_interleaved(handle, 0, (unsigned char *) OVLMEM + i, burst);
                OVLPOS -= burst;
                i += burst;
            }
            // Burst out the new sub-window position's
            //s2226_BurstWriteTransfer(handle, 0, m_ovl[idx].m_overlayMem, m_ovl[idx].m_overlayPos, 1);
            m_ovl[idx].m_overlayPos = 0;

        }
        // Signal hardware to update the overlay from the shadow registers
        // on the next start-of-frame
        s2226_write_data(handle, S2226_REG_OVAUPD, 0x01); // Set the UpdateOverlay bit
        //flush the pipe line
        s2226_write_data(handle, S2226_REG_GPREG, 0x01); // Set the UpdateOverlay bit
    //} else {
    //    // No device
    //    return MID2226_ERR_NODEVICE;
    //} 
    return 0;
} // UpdateOverlay()

/** Refresh or update the overlay to the hardware
 */
int S2226_UpdateOverlay(int input, int handle )
{
    int ret;
    int idx = get_idx_from_handle(handle);
    //#ifdef DEBUG_PROFILE
    //UINT64 st,et;
    //TCHAR dbg[200];
    //st = GetTickCount();
    //#endif
    m_ovl[idx].m_input = input;
    s2226_lock_overlay(handle);
    ret = UpdateOverlay(handle);
    s2226_unlock_overlay(handle);
    //#ifdef DEBUG_PROFILE
    //et = GetTickCount();
    //_stprintf(dbg, _T("S2226_UpdateOverlay (ms) : %d\n"), et-st);
    //OutputDebugString(dbg);
    //#endif
    return ret;
} // S2226_UpdateOverlay()


int S2226_UpdateOverlayNoInput(int handle )
{
    int rc;
    int idx = get_idx_from_handle(handle);
    int input;
    //#ifdef DEBUG_PROFILE
    //UINT64 st,et;
    //TCHAR dbg[200];
    //st = GetTickCount();
    //#endif
    // query the input
    rc = s2226_get_input(handle, &input);
    if (rc == 0) {
        m_ovl[idx].m_input = input;
    }
    // otherwise, just keep the current input.
    //    CriticalSectionBegin( m_ovl[idx].m_csOvl );
    s2226_lock_overlay(handle);
    rc = UpdateOverlay(handle);
    s2226_unlock_overlay(handle);
    //    CriticalSectionEnd( m_ovl[idx].m_csOvl );
    //#ifdef DEBUG_PROFILE
    //et = GetTickCount();
    //_stprintf(dbg, _T("S2226_UpdateOverlay (ms) : %d\n"), et-st);
    //OutputDebugString(dbg);
    //#endif

    return rc;
} // S2226_UpdateOverlay()

/** Set background overlay window
 *  @param pos        -- position and size of window
 *  @param color      -- color of back window
 *  @param regionmask -- region to output to
 *  @param group      -- 0 to OVLWINDOWS-1, or -1 to clear
 *  @return 0 on success
 */
static int SetOverlayBackground(int WinIndex, RECT *pos, int color, int regionmask, int group, int idx)
{
    if (m_ovl[idx].m_hOvl == NULL) {
        if (InitOverlay(idx) == -1)
            return -2;
    }
    
    m_ovl[idx].m_backgnd[WinIndex].pos = *pos;
    m_ovl[idx].m_backgnd[WinIndex].color = color;
    m_ovl[idx].m_backgnd[WinIndex].regionmask = regionmask;
    m_ovl[idx].m_backgnd[WinIndex].group = group; // -1 will clear it

    m_ovl[idx].m_bOvlChanged = TRUE;
    return 0;
}


///** Adds overlay text.  If overlay text already exists at that x,y position, deletes windows before adding.
// * @param xpos       -- start X position
// * @param ypos       -- start y position
// * @param pOvlText   -- pointer to overlay text
// * @param regionmask -- region mask (MID2226_REGION_MONITOR, MID2226_REGION_MPEG, MID2226_REGION_STILL)
// * @return 0 on success, -1 on too many regions, other negative value failure
// */
int S2226_OverlayText( int xpos, int ypos, overlay_text_t *pOvl, int regionmask, int handle)
{

    int WinIndex;
    int err;
    int group;
    RECT zerorect = {0,0,0,0};
    int idx = get_idx_from_handle(handle);

//    CriticalSectionBegin( m_ovl[idx].m_csOvl );  
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        if (InitOverlay(idx) == -1) {
//            CriticalSectionEnd( m_ovl[idx].m_csOvl );                
            s2226_unlock_overlay(handle);
            return -1;
        }
    }
//
//    // search existing overlays to see if anything at this position already exists. 
//    // if so, update region only!
    group = -1;
    for (WinIndex = 0; WinIndex<OVLWINDOWS ; WinIndex++) {
        if ( (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) && // Valid entry
             (((m_ovl[idx].m_overlay[WinIndex].pos.left == xpos) && ( m_ovl[idx].m_overlay[WinIndex].pos.top == ypos)) || 
              ((group!=-1) && ( m_ovl[idx].m_overlay[WinIndex].group == group)) ) )
        {
            // once found, we remove all associated overlays in the group
            if (m_ovl[idx].m_overlay[WinIndex].group != -1) {
                group = m_ovl[idx].m_overlay[WinIndex].group;
            }
            // remove it
            m_ovl[idx].m_bOvlChanged = 1;
            DelOverlayIndex(m_ovl[idx].m_hOvl, WinIndex);
            m_ovl[idx].m_overlay[WinIndex].regionmask = -1;
            m_ovl[idx].m_overlay[WinIndex].group = -1;
            SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx);
        }
    }

    err = SetOverlayTextExt( xpos, ypos, pOvl, regionmask, 0, idx);
    if( err) {
//        CriticalSectionEnd( m_ovl[idx].m_csOvl );            
        s2226_unlock_overlay(handle);
        return err;
    }
    // update overlay rectangle

    err = RegenerateOverlay(idx);
    s2226_unlock_overlay(handle);
//    CriticalSectionEnd( m_ovl[idx].m_csOvl );
    return err;
}

/** Adds/Opdate overlay text.  If overlay already exists at index WinIndex, deletes window before adding.
 * @param WinIndex   -- sub-window position to update/add text
 * @param xpos       -- new X position
 * @param ypos       -- new y position
 * @param pOvlText   -- pointer to overlay text
 * @param regionmask -- region mask (MID2226_REGION_MONITOR, MID2226_REGION_MPEG, MID2226_REGION_STILL)
 * @return 0 on success, -1 on too many regions, other negative value failure
 */
int S2226_OverlayTextIdx( int AtIndex, int xpos, int ypos, overlay_text_t *pOvl, int regionmask, int handle)
{

    int WinIndex;
    int err;
    int group;
    RECT zerorect = {0,0,0,0};
    int idx = get_idx_from_handle(handle);
//    CriticalSectionBegin( m_ovl[idx].m_csOvl );  
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        if (InitOverlay(idx) == -1) {
            s2226_unlock_overlay(handle);
//            CriticalSectionEnd( m_ovl[idx].m_csOvl );
            return -1;
        }
    }

    // search existing overlays to see if anything at this position already exists. 
    // if so, update region only!
    group = -1;
    for (WinIndex = AtIndex; WinIndex<OVLWINDOWS ; WinIndex++) {
        if ( (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) && // Valid entry
             ((WinIndex == AtIndex) || 
              ((group!=-1) && ( m_ovl[idx].m_overlay[WinIndex].group == group)) ) )
        {
            // once found, we remove all associated overlays in the group
            if (m_ovl[idx].m_overlay[WinIndex].group != -1) {
               group = m_ovl[idx].m_overlay[WinIndex].group;
            }
            // remove it
            m_ovl[idx].m_bOvlChanged = 1;
            DelOverlayIndex(m_ovl[idx].m_hOvl, WinIndex);
            m_ovl[idx].m_overlay[WinIndex].regionmask = -1;
            m_ovl[idx].m_overlay[WinIndex].group = -1;
            SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx);
        }
    }

    err = SetOverlayTextExt( xpos, ypos, pOvl, regionmask, AtIndex, idx);
    if( err) {
//        CriticalSectionEnd( m_ovl[idx].m_csOvl );            
        s2226_unlock_overlay(handle);
        return err;
    }
    // update overlay rectangle

    err = RegenerateOverlay(idx);
    s2226_unlock_overlay(handle);
//    CriticalSectionEnd( m_ovl[idx].m_csOvl );    
    return err;
}

/** Adds overlay image. If overlay already exists at that x,y position, deletes window before adding.
 * @param xpos       -- start X position
 * @param ypos       -- start y position
 * @param imageFile  -- pointer to overlay image(full path)  24bit BMP or PCX files supported only
 * @param regionmask -- output region mask 
 * @return 0 on success, -1 on too many regions, other negative value on failure
 */
 int S2226_OverlayImage( int xpos, int ypos, char *path, int regionmask, int handle)
{
    
    int WinIndex;
    int group;
    int idx = get_idx_from_handle(handle);
    //    CriticalSectionBegin( m_ovl[idx].m_csOvl );  
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        if (InitOverlay(idx) == -1) {
	  //      CriticalSectionEnd( m_ovl[idx].m_csOvl );    
            s2226_unlock_overlay(handle);
            return -1;
        }
    }

    // search existing overlays to see if anything at this position already exists. 
    // if so, delete it
    group = -1;
    for ( WinIndex=0; WinIndex < OVLWINDOWS ; WinIndex++) {
        if ( (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) && // Valid entry
             (((m_ovl[idx].m_overlay[WinIndex].pos.left == xpos) && ( m_ovl[idx].m_overlay[WinIndex].pos.top == ypos)) || 
              ((group!=-1) && ( m_ovl[idx].m_overlay[WinIndex].group == group)) ) )
        {
            break; // WinIndex = same window position
        }
        if (m_ovl[idx].m_overlay[WinIndex].regionmask < 0) {
            break; // WinIndex = free window position
        }
    }
    
    //    CriticalSectionEnd( m_ovl[idx].m_csOvl );    
    s2226_unlock_overlay(handle);
    return S2226_OverlayImageIdx( WinIndex, xpos, ypos, path, regionmask, handle);
}

/** Adds/Update overlay image. If overlay already exists at index WinIndex, deletes window before adding.
 * @param WinIndex   -- sub-window position to update/add text
 * @param xpos       -- start X position
 * @param ypos       -- start y position
 * @param imageFile  -- pointer to overlay image(full path)  24bit BMP or PCX files supported only
 * @param regionmask -- output region mask 
 * @return 0 on success, -1 on too many regions, other negative value on failure
 */
int S2226_OverlayImageIdx( int AtIndex, int xpos, int ypos, char *path, int regionmask, int handle)
{

    int WinIndex;
    int group;
    int err = 0;
    int idx = get_idx_from_handle(handle);

    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        if (InitOverlay(idx) == -1) {
            s2226_unlock_overlay(handle);
            return -1;
        }
    }

    // search existing overlays to see if anything at this position already exists. 
    // if so, delete it
    group = -1;
    for ( WinIndex=AtIndex; WinIndex < OVLWINDOWS ; WinIndex++) {
        if ( (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) && // Valid entry
             ((WinIndex == AtIndex) || 
              ((group!=-1) && ( m_ovl[idx].m_overlay[WinIndex].group == group)) ) )
        {
            // once found, we remove all associated overlays in the group
            if (m_ovl[idx].m_overlay[WinIndex].group != -1) {
                group = m_ovl[idx].m_overlay[WinIndex].group;
            }
            DelOverlayIndex(m_ovl[idx].m_hOvl, WinIndex);
            m_ovl[idx].m_overlay[WinIndex].regionmask = -1;
            m_ovl[idx].m_overlay[WinIndex].group = -1;
        }
    }
    WinIndex=AtIndex;
    if (WinIndex < OVLWINDOWS) {
        m_ovl[idx].m_overlay[WinIndex].regionmask = regionmask;
        m_ovl[idx].m_overlay[WinIndex].pos.left = xpos;
        m_ovl[idx].m_overlay[WinIndex].pos.top= ypos;
        m_ovl[idx].m_overlay[WinIndex].type = OVERLAY_TYPE_IMAGE;
        m_ovl[idx].m_overlay[WinIndex].group = -1; // No groups for images
        strncpy( m_ovl[idx].m_overlay[WinIndex].path, path, 256);
        
        err = SetOverlayImage( m_ovl[idx].m_hOvl, WinIndex, path);
        m_ovl[idx].m_overlay[WinIndex].pos.right = xpos + GetOverlayWidth( m_ovl[idx].m_hOvl, WinIndex);
        m_ovl[idx].m_overlay[WinIndex].pos.bottom = ypos + GetOverlayHeight( m_ovl[idx].m_hOvl, WinIndex);
        RegenerateOverlay(idx); // sets m_ovl[idx].m_bOvlChanged = TRUE;
    } else {
        err = -1;
    }
    s2226_unlock_overlay(handle);
    return err;
}

/** Get basic parameters of an overlay index
 * @param WinIndex   -- sub-window position to get parameters from
 * @param Type       -- Type: 1=Image 0=Text
 * @param Region     -- Overlay Region, MID2226_REGION_MONITOR | MID2226_REGION_MPEG | MID2226_REGION_STILL
 * @param Group      -- Text with common group number is kept together. (-1 indicates not used)
 * @param Xpos       -- start X position
 * @param Ypos       -- start y position
 * @param Value      -- pointer to text or image file path
 * @return 0 on success
 */
int S2226_GetOverlayIdx(int WinIndex, int *Type, int *Region, int *Group, int *Xpos, int *Ypos, char **Value, int handle)
{
    int idx = get_idx_from_handle(handle);

    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        if (InitOverlay(idx) == -1) {
            s2226_unlock_overlay(handle);
            return -1;
        }
    }
    *Type   = m_ovl[idx].m_overlay[WinIndex].type;
    *Region = m_ovl[idx].m_overlay[WinIndex].regionmask;
    *Xpos   = m_ovl[idx].m_overlay[WinIndex].pos.left;
    *Ypos   = m_ovl[idx].m_overlay[WinIndex].pos.top;
    if (*Type == 1) { // Image
        *Value = m_ovl[idx].m_overlay[WinIndex].path;
        if (*Region == -1) { // Deleted?
            *Type = -1; // Return last data
        }
    } else if (*Type == 0) { // Text
        *Value = m_ovl[idx].m_overlay[WinIndex].ovltext.text;
        if (*Region == -1) { // Deleted?
            *Type = -1; // Return last data
        }
    } else {
        *Type = -1;
//    //TODO FIXME!
//    //Note: Linux warns this is a deprecated conversion
        *Value = "";
    }
    *Group = m_ovl[idx].m_overlay[WinIndex].group;
    s2226_unlock_overlay(handle);
    return 0;
}
//
///** Set the color used for the background regions
// * @param red    -- 0 min, 255 max red
// * @param green  -- 0 min, 255 max green
// * @param blue   -- 0 min, 255 max blue
// * @return 0 on success
// */
int S2226_OverlayBackgroundColor( int red, int green, int blue, int handle )
{
    int ret=-1;
    s2226_lock_overlay(handle);
    SetBlackWindowColor(handle, RGB(red,green,blue) );
    // Signal hardware to update the overlay from the shadow registers
    // on the next start-of-frame
    ret = UpdateOverlay(handle);
    s2226_unlock_overlay(handle);
    return ret;
}

///** Change position of an overlay sub-window
// * @param WinIndex   -- index of sub-window to move
// * @param xpos       -- new X position
// * @param ypos       -- new y position
// * @return 0 on success, -1 overlay does not exist yet.
// */
 int S2226_MoveOverlay( int WinIndex, int NewX, int NewY, int handle )
{
    
    int err = 0;
    int idx = get_idx_from_handle(handle);
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        s2226_unlock_overlay(handle);
        return -1;
    }
    m_ovl[idx].m_bOvlChanged = 1;
    SetOverlayPosition(m_ovl[idx].m_hOvl, WinIndex, NewX, NewY);
    m_ovl[idx].m_overlay[WinIndex].pos.left = NewX;
    m_ovl[idx].m_overlay[WinIndex].pos.top  = NewY;
    m_ovl[idx].m_overlay[WinIndex].pos.right = NewX + GetOverlayWidth( m_ovl[idx].m_hOvl, WinIndex);
    m_ovl[idx].m_overlay[WinIndex].pos.bottom = NewY + GetOverlayHeight( m_ovl[idx].m_hOvl, WinIndex);
    if (m_ovl[idx].m_overlay[WinIndex].type == OVERLAY_TYPE_TEXT) {
        // If text, update the backgrounds
        err = RegenerateOverlay(idx);
    }
    s2226_unlock_overlay(handle);
    return err;
}

///** Delete an overlay at x,y location.
// * Does not reload or download the new image
// * @param xpos       -- X position of sub-window to delete
// * @param ypos       -- y position of sub-window to delete
// * @return 0 on success, -1 overlay does not exist yet, -2 if no overlay at that location to delete.
// */
int S2226_OverlayDelXY( int xpos, int ypos, int handle)
{
    int WinIndex;
    int err = 0;
    int group;
    RECT zerorect = {0,0,0,0};
    int idx = get_idx_from_handle(handle);
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        s2226_unlock_overlay(handle);
        return 0; // No Overlay windows to delete
    }
    
//    // search existing overlays to see if anything at this position already exists. 
//    // if so, delete it
    err = -2;
    group = -1;
    for (WinIndex = 0; WinIndex<OVLWINDOWS ; WinIndex++) {
        if ( (  m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) && // Valid entry
             (((m_ovl[idx].m_overlay[WinIndex].pos.left == xpos) && ( m_ovl[idx].m_overlay[WinIndex].pos.top == ypos)) || 
              ((group != -1) && ( m_ovl[idx].m_overlay[WinIndex].group == group)) ) )
        {
            // once found, we remove all associated overlays in the group
            m_ovl[idx].m_bOvlChanged = 1;
            group = m_ovl[idx].m_overlay[WinIndex].group;
            err = 0;
            DelOverlayIndex(m_ovl[idx].m_hOvl, WinIndex);
            m_ovl[idx].m_overlay[WinIndex].regionmask = -1;
            m_ovl[idx].m_overlay[WinIndex].group = -1;
            SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx);
        }
    }
    s2226_unlock_overlay(handle);
    return err;
}
//
///** Delete an overlay at x,y location.
// * Does not reload or download the new image
// * @param xpos       -- X position of sub-window to delete
// * @param ypos       -- y position of sub-window to delete
// * @return 0 on success, -1 overlay does not exist yet, -2 if that sub-window already unused.
// */
int S2226_OverlayDel(int WinIndex, int handle)
{
    int err = 0;
    int group;
    RECT zerorect = {0,0,0,0};
    int idx = get_idx_from_handle(handle);
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_hOvl == NULL) {
        s2226_unlock_overlay(handle);
        return 0; // No Overlay windows to delete
    }
    if ((WinIndex < 0) && (WinIndex >= OVLWINDOWS) ) {
        s2226_unlock_overlay(handle);
        return -1;
    }

    if (m_ovl[idx].m_overlay[WinIndex].regionmask < 0) { // Already empty
        err = -2;
    }
//
    DelOverlayIndex(m_ovl[idx].m_hOvl, WinIndex);
    m_ovl[idx].m_bOvlChanged = 1;
    m_ovl[idx].m_overlay[WinIndex].regionmask = -1;
    group = m_ovl[idx].m_overlay[WinIndex].group;
    m_ovl[idx].m_overlay[WinIndex].group = -1;
    SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx);

    // Remove all associated overlays in the group
    for (WinIndex = 0; WinIndex<OVLWINDOWS ; WinIndex++) {
        if ( (m_ovl[idx].m_overlay[WinIndex].regionmask >= 0) && // Valid entry
             (group != -1) && (m_ovl[idx].m_overlay[WinIndex].group == group) )
        {
            DelOverlayIndex(m_ovl[idx].m_hOvl, WinIndex);
            m_ovl[idx].m_overlay[WinIndex].regionmask = -1;
            m_ovl[idx].m_backgnd[WinIndex].group = -1;
        }
    }
    s2226_unlock_overlay(handle);
    return err;
}

///** Clear the overlay 
// */
 int S2226_ClearOverlay(int handle)
{
    unsigned int WinIndex;
    RECT zerorect = {0,0,0,0};
    int idx = get_idx_from_handle(handle);
    s2226_lock_overlay(handle);
    if (m_ovl[idx].m_arm_fw == 0) {
        m_ovl[idx].m_arm_fw = s2226_get_arm_ver(handle);
    }

    if (USE_ADDRDATA) {
        m_ovl[idx].m_overlayPos = 0;
    }

    s2226_write_data(handle, S2226_REG_OVAUPD, 0x00); // Clr the UpdateOverlay bit
    
    // Remove backgrounds
    for ( WinIndex=0; WinIndex < OVLWINDOWS ; WinIndex++) {
        //SetBlackWindowPosition(handle, WinIndex, 0,0,0,0,0);
        // Just write the byte with the region bits
        WRITE_INDEXED_BYTE_DLL(USE_ADDRDATA, S2226_REG_BWLBMSB + (WinIndex*8), 0 );
        SetOverlayBackground(WinIndex, &zerorect, 0, -1, -1, idx); // Mark as deleted
    }
    
    // Remove sub-windows
    for (WinIndex=0; WinIndex < OVLWINDOWS ; WinIndex++) {
        if( m_ovl[idx].m_overlay[WinIndex].regionmask >= 0 ) {
            // clear unused regions
            SET_OVERLAY_WINDOW_POSITION_DLL(USE_ADDRDATA, WinIndex, 0,0,0,0,0);
            //DelOverlayIndex(WinIndex); // Not needed here. DestroyOverlay() called below
            m_ovl[idx].m_overlay[WinIndex].regionmask = -1; // Mark as deleted
        }
    }
    if (USE_ADDRDATA) {
        // Burst Write all the background/window position data
        //s2226_BurstWriteTransfer(handle, 0, m_ovl[idx].m_overlayMem, m_ovl[idx].m_overlayPos, 1);
        {
            int i = 0;
            int burst;
            // Burst out the new sub-window position's
            while (OVLPOS > 0) {
                burst = OVLPOS > 498 ? 498 : OVLPOS;
                s2226_write_addr_data_interleaved(handle, 0, (unsigned char *) OVLMEM + i, burst);
                OVLPOS -= burst;
                i += burst;
            }
        }
        m_ovl[idx].m_overlayPos = 0;
    }
    // Signal hardware to update the overlay from the shadow registers
    // on the next start-of-frame
    s2226_write_data(handle, S2226_REG_OVAUPD, 0x01); // Set the UpdateOverlay bit

    // erase background windows
    if (m_ovl[idx].m_hOvl)
    {
        DestroyOverlay(m_ovl[idx].m_hOvl);
        m_ovl[idx].m_hOvl = NULL;
    }
    s2226_unlock_overlay(handle);
    return 0;
}
//
///** clears all overlays for specified region. Use S2226_ClearOverlay to erase overlays.  This function
// *  only clears the overlay by region
// *  @param regionmask output region mask
// */
int S2226_ClearOverlayRegion( int regionmask, int handle)
{
    unsigned int WinIndex;
    int idx = get_idx_from_handle(handle);
    s2226_lock_overlay(handle);
    for (WinIndex=0; WinIndex < OVLWINDOWS ; WinIndex++) {
        m_ovl[idx].m_bOvlChanged = 1;
        m_ovl[idx].m_overlay[WinIndex].regionmask &= ~regionmask; /*lint !e502: expected unsigned type */
        m_ovl[idx].m_backgnd[WinIndex].regionmask &= ~regionmask; /*lint !e502: expected unsigned type */
    }
    UpdateOverlay(handle);
    s2226_unlock_overlay(handle);
    return 0;
}
//
///** set all overlays for specified region.This function
// *  only sets regionmask for all overlays
// *  @param regionmask output region mask
// */
int S2226_SetOverlayRegion( int regionmask, int handle)
{
    unsigned int WinIndex;
    int idx = get_idx_from_handle(handle);
    s2226_lock_overlay(handle);
    for (WinIndex=0; WinIndex < OVLWINDOWS ; WinIndex++) {
        m_ovl[idx].m_overlay[WinIndex].regionmask |= regionmask;
        m_ovl[idx].m_backgnd[WinIndex].regionmask |= regionmask;
    }
    m_ovl[idx].m_bOvlChanged = 1;
    UpdateOverlay(handle);
    s2226_unlock_overlay(handle);
    return 0;
}


//////////////////////////////////////////////////////////////////////////
// Snapshot
//////////////////////////////////////////////////////////////////////////

/** Get snapshot data from hardware only.  This function does no processing on the image, it just
 * retrieves a pointer to the raw fields(field 1 will be null for 720p format) in YCrCb format
 * @param image is a pointer to pre-allocated space for image
 * @param size is the size of image array above
 * @param pImage is a pointer to the returned raw image fields
 * @param freezetime is the time, in milli-sec to freeze the image on the monitor. (a minimum value greater than the video frame rate is required)
 * @param wait wait=1 will wait if operations pending(board locked), 
 *             wait=0 returns error if board busy
 * @param board ID of 2226 if multiple boards installed.
 * @return number of bytes read (image size) or 
 *    MID2226_SNAP_ERR_UNSPECIFIED -1
 *    MID2226_SNAP_ERR_BUFSIZE     -2  'Size' of buffer provided is too small for snapshot.
 *    MID2226_SNAP_ERR_BIST        -3  Built in self test failed. Hardware not ready.
 *    MID2226_SNAP_ERR_SYNCH       -4  Aquisition's synchronization loop timed out.
 *    MID2226_SNAP_ERR_CRITSEC     -5  Aquisition's critical section already locked, and user doesn't want to wait for it.
 *    MID2226_SNAP_ERR_PARTBUF     -6  Didn't recieve a full frame of data.
 */
int S2226_SnapshotRaw( BYTE *image, int size, image_raw_t *pImage, int freezetime, int wait, int handle)
{
    int idx = get_idx_from_handle(handle);
    int byteCount = m_ovl[idx].m_imageHeight * m_ovl[idx].m_imageWidth * 3;
    int res;
    if (size < byteCount)
      return -2;//MID2226_SNAP_ERR_BUFSIZE; // not enough space allocated
    //if (S2226WaitForBist(handle ) == FALSE) { // Wait for PLL/DDR test
    //   return MID2226_SNAP_ERR_BIST;
    //}
    res = GetSnapshot(handle, image, &byteCount, freezetime, wait);
    if( res < byteCount) { // Should be equal
        if (res == -2) {
	  return -4;//MID2226_SNAP_ERR_SYNCH; // Aquisition's synchronization loop timed out.
        }
        if (res == -1) {
	  return -5;//MID2226_SNAP_ERR_CRITSEC; // Aquisition's critical section already locked, and user doesn't want to wait for it.
        }
        // else
        return -6;//MID2226_SNAP_ERR_PARTBUF; // Didn't recieve a full frame of data.
    }

    pImage->f0 = image;
    pImage->size0 = m_ovl[idx].m_field0Height * 3 * m_ovl[idx].m_imageWidth;
    pImage->f1 = image + m_ovl[idx].m_field0Height * 3 * m_ovl[idx].m_imageWidth;
    pImage->size1 = m_ovl[idx].m_field1Height * 3 * m_ovl[idx].m_imageWidth;
    return res;
}

#ifdef SENSORAY_OS_LINUX
#ifdef __cplusplus
}
#endif
#endif

/** Get snapshot to memory function.  Retrieves processed image into memory.
  * Image is converted to RGB (1 byte per color) and then the fields are merged 
  * using the algorithm set by S2226_SetMergeMethod().
  * @param image IN/OUT pointer to retrieved snapshot.  
  * @param size is the size of the image buffer.
  * @param freezetime is the time, in milli-sec to freeze the image on the monitor. (a minimum value greater than the video frame rate is required)
  * @param wait wait =1 will wait if operations pending(board locked), 
  *                  wait=0 returns error if board busy
  * @param pRetW pointer to image width (in Pixels).  Modify this to a new value if the returned image width is different. Initial value *pRetW = w/3
  * @param pRetH pointer to image height. Modify this to a new value if the returned image height is different. Initial value *pRetH = h0 + h1
  * @param board ID of 2226 if multiple boards installed.
  * @return number of bytes read (image size) or 
  *    MID2226_SNAP_ERR_UNSPECIFIED -1
  *    MID2226_SNAP_ERR_BUFSIZE     -2  'Size' of buffer provided is too small for snapshot.
  *    MID2226_SNAP_ERR_BIST        -3  Built in self test failed. Hardware not ready.
  *    MID2226_SNAP_ERR_SYNCH       -4  Aquisition's synchronization loop timed out.
  *    MID2226_SNAP_ERR_CRITSEC     -5  Aquisition's critical section already locked, and user doesn't want to wait for it.
  *    MID2226_SNAP_ERR_PARTBUF     -6  Didn't recieve a full frame of data.
  *    MID2226_SNAP_ERR_DVBUF       -7  Failed to get current frame buffer from DV
  *    MID2226_SNAP_ERR_DVGRAB      -8  The DV frame grabber was not setup
  *    MID2226_SNAP_ERR_USERERR     -9  Error returned from user's merge function
  */
static int SnapshotToMem( BYTE *image, int size, int freezetime, int wait, int *pRetW, int *pRetH, int handle )
{
    BYTE *image_yuv;
    BYTE *image_rgb;
    BYTE *image_merge;
    BYTE *f0, *f1;
    long cbBuffer;
    int  byteCount;
    int idx = get_idx_from_handle(handle);
    // From Rp.219 Definition
    //   Gry 75W Yel Cy  Gr  Mg  Red Blu Gry
    //   --- --- --- --- --- --- --- --- ---
    //Y  104 180 168 145 133  63  51  28 104
    //Pr 128 128 136  44  52 204 212 120 128
    //Pb 128 128  44 147  63 193 109 212 128
    //Y   Pr  Pb 
    //BYTE  Test[] = {
    //   104, 128, 128, // Gray
    //   180, 128, 128, // 75% White (Light Grey)
    //   168, 136,  44, // Yelllow
    //   145,  44, 147, // Cyan
    //   133,  52,  63, // Green
    //    63, 204, 193, // Magenta
    //    51, 212, 109, // Red
    //    28, 120, 212, // Blue
    //   104, 128, 128  // Gray
    //};

    // Make sure width/height set
    if (SetFieldDims(idx) != 0)
        return -1;
    // Setup Buffer for merge routines
    byteCount = m_ovl[idx].m_imageHeight * m_ovl[idx].m_imageWidth * 3;  // currently need some extra space
    *pRetH    = m_ovl[idx].m_imageHeight; // m_ovl[idx].m_field0Height + m_ovl[idx].m_field1Height
    *pRetW    = m_ovl[idx].m_imageWidth;
    
    if (size < byteCount) {
      return -2;//MID2226_SNAP_ERR_BUFSIZE; // 'Size' of buffer provided is too small for snapshot.
    }
    
    // image should be pre-allocated
    image_yuv   = image; 
    image_rgb   = image; 
    
    // merge image, however, is dynamically allocated
    image_merge = (BYTE *) malloc( size + (m_ovl[idx].m_imageWidth * 3));  // allocate one extra line just in case
    if (image_merge == NULL) {
        return -1; // Could not allocate memory
    }

        //if (S2226WaitForBist(handle ) == FALSE) { // Wait for PLL/DDR test
        //  free( image_merge);
        //  return MID2226_SNAP_ERR_BIST; // Built in self test failed. Hardware not ready.
        //}
        // Get snapshot from hardware
        cbBuffer = GetSnapshot(handle, image_yuv, &byteCount, freezetime, wait);
        if( cbBuffer < byteCount) { // Should be equal
            free( image_merge);
            if (cbBuffer == -2) {
	      return -4;//MID2226_SNAP_ERR_SYNCH; // Aquisition's synchronization loop timed out.
            }
            if (cbBuffer == -1) {
	      return -5;//MID2226_SNAP_ERR_CRITSEC; // Aquisition's critical section already locked, and user doesn't want to wait for it.
            }
            // else
            return -6;//MID2226_SNAP_ERR_PARTBUF; // Didn't recieve a full frame of data.
        }
        
        //memcpy(image, Test, sizeof(Test));
        
        // We have the image data in YCrCb from hardware. Convert it to RGB
        // image_yuv is the same as image_rgb for efficiency here in pointer == out pointer
        //
        // NOTE: HD cameras use a different Color space definition
        //
	switch (m_ovl[idx].m_input) {
	case INPUT_SDI_1080I_50:
	case INPUT_SDI_1080I_60:
	case INPUT_SDI_1080I_5994:
	case INPUT_SDI_1080I_60_CB:
	case INPUT_SDI_1080I_50_CB:
	case INPUT_H51_HD_1080I_60:
	case INPUT_H51_HD_1080I_50:
	case INPUT_H51_HD_1080I_5994:
	case INPUT_SDI_720P_5994:
	case INPUT_SDI_720P_60:
	case INPUT_SDI_720P_60_CB:
	case INPUT_H51_HD_720P_60:
	case INPUT_H51_HD_720P_5994:
	case INPUT_SDI_720P_50:
	case INPUT_SDI_720P_50_CB:
	case INPUT_H51_HD_720P_50:	  
	  // High Definition camera. Use BT709 standard colorspace.
	  YCrCb2RGB_BT709((unsigned char *)image_yuv, (unsigned char *)image_rgb, m_ovl[idx].m_imageHeight, m_ovl[idx].m_imageWidth);
	  break;
	default:
	  // Standard Def.
	  YCrCb2RGB_BT601((unsigned char *)image_yuv, (unsigned char *)image_rgb, m_ovl[idx].m_imageHeight, m_ovl[idx].m_imageWidth);
	    break;
        }
        // set the field pointers
        f0 = image_rgb;
        f1 = image_rgb + m_ovl[idx].m_field0Height * 3 * m_ovl[idx].m_imageWidth;
	
    switch( m_ovl[idx].m_mergeType)
        {
        case MID2226_FIELDALG_DUP:
	  dup_fields((unsigned char *)image_merge, byteCount,
		     (unsigned char *)f0, m_ovl[idx].m_field0Height,
		     (unsigned char *)f1, m_ovl[idx].m_field1Height, m_ovl[idx].m_imageWidth*3,
                        pRetW, pRetH);
            break;
        case MID2226_FIELDALG_MERGE:
	  merge_fields((unsigned char *) image_merge, byteCount, (unsigned char *)f0, m_ovl[idx].m_field0Height,
                        (unsigned char *)f1, m_ovl[idx].m_field1Height, m_ovl[idx].m_imageWidth*3,
                        pRetW, pRetH);
            break;
        case MID2226_FIELDALG_INTER:
	  inter_fields((unsigned char *) image_merge, byteCount, (unsigned char *)f0, m_ovl[idx].m_field0Height,
                        (unsigned char *)f1, m_ovl[idx].m_field1Height, m_ovl[idx].m_imageWidth*3,
                        pRetW, pRetH);
            break;
        default: // no merge
            memcpy( image_merge, image_rgb, size); // copy size bytes since image size may have been modified
            break;
        }
        // copy merged result to image
        memcpy( image, image_merge, size); // copy size bytes since image size may have been modified
        
    // Use returned sizes to calculate return value
    byteCount = *pRetH * *pRetW * 3; // same as orig byteCount if RetW/RetH not modified by user

    free( image_merge);
    return byteCount;
} // COvlMid::SnapshotToMem

#ifdef SENSORAY_OS_LINUX
#ifdef __cplusplus
extern "C" {
#endif
#endif

///** Get snapshot to memory function.  Retrieves processed image into memory.
//  * Image is converted to RGB (1 byte per color) and then the fields are merged 
//  * using the algorithm set by S2226_SetMergeMethod().
//  * @param image IN/OUT pointer to retrieved snapshot.  
//  * @param size is the size of the image buffer.
//  * @param freezetime is the time, in milli-sec to freeze the image on the monitor. (a minimum value greater than the video frame rate is required)
//  * @param wait wait =1 will wait if operations pending(board locked), 
//  *                  wait=0 returns error if board busy
//  * @param board ID of 2226 if multiple boards installed.
//  * @return number of bytes read (image size) or 
//  *    MID2226_SNAP_ERR_UNSPECIFIED -1
//  *    MID2226_SNAP_ERR_BUFSIZE     -2  'Size' of buffer provided is too small for snapshot.
//  *    MID2226_SNAP_ERR_BIST        -3  Built in self test failed. Hardware not ready.
//  *    MID2226_SNAP_ERR_SYNCH       -4  Aquisition's synchronization loop timed out.
//  *    MID2226_SNAP_ERR_CRITSEC     -5  Aquisition's critical section already locked, and user doesn't want to wait for it.
//  *    MID2226_SNAP_ERR_PARTBUF     -6  Didn't recieve a full frame of data.
//  *    MID2226_SNAP_ERR_DVBUF       -7  Failed to get current frame buffer from DV
//  *    MID2226_SNAP_ERR_DVGRAB      -8  The DV frame grabber was not setup
//  *    MID2226_SNAP_ERR_USERERR     -9  Error returned from user's merge function
//  */
int S2226_SnapshotToMem( BYTE *image, int size, int freezetime, int wait, int handle)
{
    int rc;
    int RetW;
    int RetH;
    rc = SnapshotToMem(image, size, freezetime, wait, &RetW, &RetH, handle);
    return rc;
}


/** Takes a snapshot and saves to file in filename
  * @param szFilename is the fully qualified file + path( without extension)
  * @param filetype is the file type to save( see mid2226const.h: MID2226_FILE_JPEG and/or MID2226_FILE_BMP)
  * @param freezetime is the time, in milli-sec to freeze the image. (minimum value greater than the video frame rate is required)
  * @param wait  wait =1 will wait if operations pending(board locked), 
  *                  wait=0 returns error if board busy
  * @return number of bytes read (image size) or 
  *    MID2226_SNAP_ERR_UNSPECIFIED -1
  *    MID2226_SNAP_ERR_BUFSIZE     -2  'Size' of buffer provided is too small for snapshot.
  *    MID2226_SNAP_ERR_BIST        -3  Built in self test failed. Hardware not ready.
  *    MID2226_SNAP_ERR_SYNCH       -4  Aquisition's synchronization loop timed out.
  *    MID2226_SNAP_ERR_CRITSEC     -5  Aquisition's critical section already locked, and user doesn't want to wait for it.
  *    MID2226_SNAP_ERR_PARTBUF     -6  Didn't recieve a full frame of data.
  *    MID2226_SNAP_ERR_DVBUF       -7  Failed to get current frame buffer from DV
  *    MID2226_SNAP_ERR_DVGRAB      -8  The DV frame grabber was not setup
  *    MID2226_SNAP_ERR_USERERR     -9  Error returned from user's merge function
  */
int S2226_SnapshotToFile(char * pFile, int filetype, int freezetime, int wait, int handle)
{
    BYTE *image;
    image_info_t info;
    int retval;
    int byteCount;
    int RetW;
    int RetH;
    int idx = get_idx_from_handle(handle);
    // Make sure width/height set
    if (SetFieldDims(idx) != 0)
        return -1;
    // set the field parameter for the new input type
    //pDev->m_ovl[idx].m_ovl->SetFieldDims( (MID2226_INPUT) pDev->m_ovl[idx].m_ovl->m_ovl[idx].m_input, (MID2226_VIDSYS) pDev->m_ovl[idx].m_ovl->m_ovl[idx].m_vidsys); // Set Height/Width for DV. (Already set if in 2226 mode)
    byteCount = m_ovl[idx].m_imageHeight * m_ovl[idx].m_imageWidth * 3;

    image = (BYTE *) malloc (byteCount);
    if (image == NULL) {
        return -1;
    }
    retval = SnapshotToMem( image, byteCount, freezetime, wait, &RetW, &RetH, handle);
    info.height = RetH; // same as pDev->m_ovl[idx].m_ovl->m_ovl[idx].m_imageHeight if not modified by user
    info.width  = RetW; // same as pDev->m_ovl[idx].m_ovl->m_ovl[idx].m_imageWidth  if not modified by user
    info.stride = info.width *3; 
    if( retval >= 0)
    {
#if 0
        if( filetype & MID2226_FILE_JPEG)
        {
            image_to_jpg( image, pFile, &info, qual);
        }
#endif
        if( filetype & MID2226_FILE_BMP)
        {
	  save_image_uncompressed((unsigned char *)image, pFile, &info, 0);
        }
        if( filetype & MID2226_FILE_PPM)
        { 
	  save_image_uncompressed((unsigned char *) image, pFile, &info, 1);
        }   
    }
    free( image);
    return(retval);
}

int S2226_SetMergeMethod( MID2226_FIELDALG merge_method, int handle)
{
    int idx = get_idx_from_handle(handle);
    m_ovl[idx].m_mergeType = merge_method;
    return 0;
}


