// ovlmid.h : Overlay routines for handeling eight individual text and graphic
// elements.
// Also contains snapshots routines
// 

#if !defined(__OVLMID_H)
#define __OVLMID_H

#if _MSC_VER > 1000
 #pragma once
#endif // _MSC_VER > 1000

#include <string.h>


#define TRUE 1
#define FALSE 0
#define BYTE char
#define WORD short
#define DWORD int
#define MAX_PATH 260
#define RGB(r,g,b) ((int)(((unsigned char)(r)|((unsigned short)((unsigned char)(g))<<8))|(((unsigned short)(unsigned char)(b))<<16)))
#define MIN(x,y) (((x)<(y)) ? (x) : (y))
#define MAX(x,y) (((x)>(y)) ? (x) : (y))


typedef struct {
	int left;
	int right;
	int top;
	int bottom;
} RECT;

// background black window
typedef struct {
    int color;
    int regionmask;
    RECT pos;
    int group; // group belongs to
} backwnd_t; 

typedef struct {
    char text[256];
    char fontfile[256]; 
    int  fontsize;  
    int  txtcolor;
    int  backcolor; 
    char reserved[10];
} overlay_text_t;

#define OVERLAY_TYPE_TEXT   0
#define OVERLAY_TYPE_IMAGE  1

typedef struct {
    int  regionmask; // (aka enable)
    RECT pos;
    int  group; // group belongs to
    overlay_text_t ovltext; // overlay text
    char path[256]; // image path
    int  type;  // normally 0 - text, 1 image
} overlay_t;


int S2226_OverlayText( int xpos, int ypos, overlay_text_t *pOvl, int regionmask, int handle);
// AtIndex (0-7)
int S2226_OverlayTextIdx( int AtIndex, int xpos, int ypos, overlay_text_t *pOvl, int regionmask, int handle);

int S2226_OverlayImage( int xpos, int ypos, char *path, int regionmask, int handle);
// AtIndex (0-7)
int S2226_OverlayImageIdx( int AtIndex, int xpos, int ypos, char *path, int regionmask, int handle);

int S2226_OverlayBackgroundColor( int red, int green, int blue, int m_handle );

int S2226_GetOverlayIdx(int WinIndex, int *Type, int *Region, int *Group, int *Xpos, int *Ypos, char **Value, int handle);
int S2226_MoveOverlay( int WinIndex, int NewX, int NewY, int handle );

// physically updates the overlay at specified index
// UpdateOverlayNoInput may be used from separate threads
// without knowing the input set on the other thread
// DEPRECATED. kept for backward compat only.
// Please use S2226_UpdateOverlayNoInput

int S2226_UpdateOverlay(int input, int handle );

int S2226_UpdateOverlayNoInput(int handle);


int S2226_OverlayDelXY( int xpos, int ypos, int handle);
int S2226_OverlayDel(int WinIndex, int handle);


int S2226_SetOverlayRegion( int regionmask, int handle);
int S2226_ClearOverlayRegion( int regionmask, int handle);

int S2226_ClearOverlay(int handle);
int S2226_OverlayRegenerate(int handle);

/** Raw image structure
 *  @param f0 pointer to field 0
 *  @param size0 size of field 0
 *  @param f1 pointer to field 1 or null if not present
 *  @param size1 size of field 1
 */
typedef struct 
{
    BYTE *f0;
    int  size0;
    BYTE *f1;
    int  size1;
} image_raw_t;
// for raw capture, fields are separated.
int S2226_SnapshotRaw( BYTE *image, int size, image_raw_t *pImage, int freezetime, int wait, int handle);

int S2226_SnapshotToFile( char *szFilename, int filetype, int freezetime, int wait, int handle);
int S2226_SnapshotToMem( BYTE *image, int size, int freezetime, int wait, int handle);
#ifdef __cplusplus
enum MID2226_FIELDALG
#else
typedef enum
#endif
{
    MID2226_FIELDALG_NONE  = 0,   // no field manipulation is done
    MID2226_FIELDALG_DUP   = 1,   // duplicate field 0 
    MID2226_FIELDALG_MERGE = 2,  // merge field 0 and 1
    MID2226_FIELDALG_INTER = 3,  // interpolate between field 0
#ifdef __cplusplus
};
#else
} MID2226_FIELDALG;
#endif

// merge allows full interlaced field merge control for SnapshotToFile and 
//
int S2226_SetMergeMethod( MID2226_FIELDALG merge_method, int m_handle);

// Image file save bitmask types 


//#define MID2226_FILE_JPEG           1 // not supported yet(uses JPEG libraries, not hardware JPEG capture)

#define MID2226_FILE_BMP            2 // True hardware capture, not SW decoded
#define MID2226_FILE_PPM            4 // True hardware capture, not SW decoded

#endif // !defined(__OVLMID_H)
